/**
 * Electronic Ticketing System (ETS)
 * Copyright (C) 2006, Magnus Egeberg
 * 
 * This file is part of ETS.
 * 
 * ETS is a program for using the mobile phone as a contactless electronic ticket.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 */
import javax.microedition.lcdui.*;

/**
 * BusyAlert is used to notify users of the process of an ongoing task
 * 
 * @author Magnus Egeberg
 * @since 1.0
 */
public class BusyAlert extends Alert{

       private Gauge gauge;
       private TicketingSystemMIDlet ts;

       /**
        * Sets the title of the alert
        * 
        * @param title The alert title
        * @param ts The MIDlet
        */
       public BusyAlert(String title, TicketingSystemMIDlet ts) {
           super(title);
           this.ts = ts;
       }

       /*
        * Creates the alert
        * 
        * @param text The text to be presented
        */
       public void doAlert(String text) {
    	   ts.trace("BusyALert: doAlert" + text);
           gauge = new Gauge(null, false, Gauge.INDEFINITE, Gauge.CONTINUOUS_RUNNING);

           setIndicator(gauge);
           setTimeout(1000*120);
           setText(text);
           showAlert(this);
       }

       /**
        * Dismisses the alert
        */
       public void dismiss() {
    	   ts.trace("BusyAlert: dismiss");
           ts.display.setCurrent(ts.mainForm);
       }

       /**
        * Changes the alert text
        * 
        * @param text The new alert text
        */
       public void setText(String text) {
    	   ts.trace("BusyAlert setText");
    	   setString(text);
    	   ts.manager.setBacklight(1000 * 3);
       }
	
       /**
        * Makes the alert visible on the deveice screen
        * 
        * @param a The alert to make visible
        */
       public void showAlert(Alert a) {
    	  ts.trace("BusyAlert: showAlert");
          ts.display.setCurrent(a);
          ts.manager.setBacklight(1000 * 3);
       }
}
