/**
 * Electronic Ticketing System (ETS)
 * Copyright (C) 2006, Magnus Egeberg
 * 
 * This file is part of ETS.
 * 
 * ETS is a program for using the mobile phone as a contactless electronic ticket.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 */
import javax.microedition.lcdui.*;

/**
 * ConfirmationAlert is used to notify users and require user feedback to proceed
 * 
 * @author Magnus Egeberg
 * @since 1.0
 */
public class ConfirmationAlert extends Alert implements CommandListener{
	
	//Command text
	private final String CMD_EXIT = "Exit";
	private final String CMD_MENU = "Menu";
	private final String CMD_OK = "Ok";
	private final String CMD_ERROR_UNRECOVERABLE = "Unrecoverable error!\nTrace on next screen...";
	
	//Commands
	private Command cmdExit;
	private Command cmdMenu;
	private Command cmdOk;
	
	//The MIDlet
	private TicketingSystemMIDlet ts;
	
	/**
	 * Creates the alert
	 * 
	 * @param title The title of the alert
	 * @param ts The MIDlet
	 */
	
	public ConfirmationAlert(String title, TicketingSystemMIDlet ts){
		super(title);
		this.ts = ts;
		cmdExit = new Command(CMD_EXIT, Command.EXIT, 1);
		cmdMenu = new Command(CMD_MENU, Command.SCREEN, 2);
		addCommand(cmdExit);
		addCommand(cmdMenu);
		setCommandListener(this);
	}
	
	/**
	 * Adds mesage to the alert
	 * 
	 * @param text The text to be provided to the user
	 */
	 public void setText(String text) {
		 ts.trace("confirmationALert: setText");
  	     setString(text);
  	     ts.manager.setBacklight(1000 * 3);
     }
	
	 /**
	  * Performs actions based on user input
	  * 
	  * @param c The command
	  * @param d The displayable
	  */
	 public void commandAction(Command c, Displayable d) {
		 ts.trace("confirmationALert: commandAction");
		 if (c == cmdExit)
			 ts.destroyApp(false);
		 if (c == cmdMenu)
			 ts.display.setCurrent(ts.mainForm);
		 if (c == cmdOk)
			 ts.display.setCurrent(ts.errorForm);
	 }
	 
	 /**
	  * Removes the menu command when an error occurs
	  * Notifies the user of the error
	  */
	 public void error(){
		 ts.trace("confirmationALert: error");
		 removeCommand(cmdMenu);
		 cmdOk = new Command(CMD_OK, Command.OK, 2);
		 addCommand(cmdOk);
		 setCommandListener(this);
		 setText(CMD_ERROR_UNRECOVERABLE);
	     ts.display.setCurrent(this, ts.errorForm);
	     ts.manager.setBacklight(1000 * 3);
	 }
	 
}
