/**
 * Electronic Ticketing System (ETS)
 * Copyright (C) 2006, Magnus Egeberg
 * 
 * This file is part of ETS.
 * 
 * ETS is a program for using the mobile phone as a contactless electronic ticket.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 */
import java.io.*;
import java.util.Calendar;

import javax.microedition.io.*;

/**
 * HTTPConnection is used to order tickets in a prototype
 * 
 * @author Magnus Egeberg
 * @since 1.0
 */
public class HTTPConnection implements Runnable{
	
	private TicketingSystemMIDlet ts;
	private String baseUrl;
	private String value;
	
	/**
	 * Starts the thread to order the ticket
	 * 
	 * @param ts The MIDlet
	 * @param value The purchase ammount
	 */
	public HTTPConnection(TicketingSystemMIDlet ts, String value){ 
		
		//The url of the server providing tickets
		baseUrl ="http://baltazar.nta.no/kannel/sendsms?username=kannel&password=deetpo&to=%2B4797679590&udh=%06%05%04%15%B3%15%B3&coding=1&text=%FF%AA";
		this.ts = ts;
		this.value = value;
		
		new Thread(this).start();
		
	}
	
	/**
	 * Puts together the complete url for purchase
	 */
	public void run(){
		
		String date = getDate();
		String url = baseUrl + date + value;
		try {
		      invoke(url);
		}catch (IOException e) {
		      ts.trace(e.toString());
		}
		finally{
			 sleep();
		 }
	}
	
	/**
	 * Connects to the server
	 * 
	 * @param url The url of the server providing tickets
	 * @see HTTPConnection
	 * @see InputStream
	 * @see StringBuffer
	 * @throws IOException
	 */
	private void invoke(String url)throws IOException{
		HttpConnection c = null;
	    InputStream is = null;
	    StringBuffer b = new StringBuffer();
	    try {
	      c = (HttpConnection)Connector.open(url);
	      is = c.openDataInputStream();
	      int ch;

	      //receive response.
	      while ((ch = is.read()) != -1) {
	        b.append((char) ch);
	      }
	    } finally {
	      if(is!= null) {
	        is.close();
	      }
	      if(c != null) {
	        c.close();
	      }
	    }
	    //Send the response to the MIDlet
	    ts.purchase(b.toString());
	}
	
	/**
	 * Gets the current date
	 * 
	 * @return Stringrepresentation of the current data interperated as bytes (0x00)
	 */
	public String getDate(){
		String dHX, mHX, yHX;
		
		Calendar today = Calendar.getInstance();
		
		int day = today.get(Calendar.DATE);
		if(day<16)
			dHX = "0"+Integer.toHexString(day);
		else
			dHX = Integer.toHexString(day);
		
		int month = 1 + today.get(Calendar.MONTH);
		mHX = "0" + Integer.toHexString(month);
		
		int year = today.get(Calendar.YEAR) - 2000;
		if(year<16)
			yHX = "0"+Integer.toHexString(year);
		else
			yHX = Integer.toHexString(year);
		
		String date = "%"+dHX+"%"+mHX+"%"+yHX;
		return date;
	}
	
	/**
	 * Puts the current thread to sleep
	 */
	private void sleep(){
		boolean sleep = true;
		while(sleep){
			try{
			Thread.currentThread().sleep(10 * 1000);
			} catch(InterruptedException ie){
				
			}
		}
	}

}