/**
 * Electronic Ticketing System (ETS)
 * Copyright (C) 2006, Magnus Egeberg
 * 
 * This file is part of ETS.
 * 
 * ETS is a program for using the mobile phone as a contactless electronic ticket.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 */

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import javax.microedition.lcdui.*;
import javax.microedition.io.Connector;
import javax.wireless.messaging.*;

import com.nokia.nfc.shared.ui.DeviceManager;

/**
 * TicketingSystemMIDlet is responsible for the life cycle of the MIDlet.
 * It is responsible for receiving incoming sms.
 * It offers the menu selections to the user.
 * 
 * @author Magnus Egeberg
 * @since 1.0
 */
public class TicketingSystemMIDlet extends MIDlet implements MessageListener, ItemCommandListener, Runnable{
	
	//Main menu items
	private final String MENU_MAIN_TITLE = "Main Menu";
	private final String MENU_CURRENT_CREDIT = "Current ticket\n";
	private final String MENU_LAST_PURCHASE = "Last purchase\n";
	private final String MENU_BUY_TICKET = "Order ticket\n";
	private final String MENU_CMD_EXIT = "Exit";
	private final String MENU_CMD_SELECT = "Select";
	
	//Order menu
	private final String ORDER_MAIN_TITLE = "Purchase";
	private final String ORDER_CMD_BACK = "Back";
	private final String ORDER_CMD_SELECT = "Select";
	private final String ENDING_KR = ",- kr.\n";
	
	//Error form
	private final String ERROR_TITLE = "Error";
	private final String ERROR_DATA = "Data";
	private final String ERROR_SMS = "SMS :";
	private final String ERROR_SMS_TICKET = "ticket";
	private final String ERROR_SMS_NOT_A_TICKET = "Not a ticket";
	private final String ERROR_RECEIVED_SMS_TXT = "SMS coded as text";
	
	//Busy alert
	private final String BUSY_TITLE = "Busy";
	private final String BUSY_RECEIVING_DATA = "Receiving data...";
	private final String BUSY_TICKET_RECEIVED = "Ticket received.";
	private final String BUSY_CONNECTING_TO_CHIP = "Connecting to chip...";
	
	//Confirmation alert title
	private final String CONFIRMATION_ERROR_TITLE = "Error";
	
	//Device Manager
	public DeviceManager manager;
	public static int[][] vibeSequence ={{DeviceManager.VIBRATE, 100},
			    					     {0, 100},
			    						 {DeviceManager.VIBRATE, 100}};
	
	//The main displayable
	public Form mainForm;
	private ImageItem nfcLogo;
	private Command mainSelectCmd;
	private Command mainExitCmd;
	private StringItem[] mainView;
	
	//The order form
	public Form orderForm;
	private Command orderSelectCmd;
	private Command orderBackCmd;
	private StringItem[] orderView;
	
	//The error form
	public Form errorForm;
	
	//Identifies selection in the main menu
	public int info;
	
	//The alerts
	public BusyAlert busy;
	public BusyAlert purchasing;
	
	//SMS connection
	public String smsPort = null;
	private static String SmsPortAttribute = "sms-port";
	private static String DefaultSmsPort = "7552";
	private MessageConnection msgConn = null;
	private byte[] buf= null;
	
	//**General
	public Display display;
	public boolean handle_error = false;
	private SecureChipHandler secureChip;
	
	//Debug
	public StringItem tracelines;
	
	/**
	 * Empty Constructor
	 */
	public TicketingSystemMIDlet(){
		
	}
	
	/**
	 * Creates the menus.
	 * Opens a SecureChipHandler
	 * Starts a thread
	 * 
	 * @throws MIDletStateChangeException
	 */
	protected void startApp() throws MIDletStateChangeException {
		display = Display.getDisplay(this);
		new Thread(this).start();
		manager = DeviceManager.newInstance(display);
		
		//Create main form
		mainForm = new Form(MENU_MAIN_TITLE);
		
		//Add logo to main form
		nfcLogo = new ImageItem(null, createImage("/res/NFC.PNG"), ImageItem.LAYOUT_CENTER | ImageItem.LAYOUT_NEWLINE_AFTER, "Near Field Communication");
		mainForm.append(nfcLogo);
		
		//Create main form commands
		mainSelectCmd = new Command(MENU_CMD_SELECT, Command.ITEM, 1);
        mainExitCmd = new Command(MENU_CMD_EXIT, Command.EXIT, 1);
        
        //Create and add the buttons shown in the main form.
        mainView = new StringItem[3];
        mainView[0] = new StringItem(null, MENU_CURRENT_CREDIT, Item.BUTTON);
        mainView[1] = new StringItem(null, MENU_LAST_PURCHASE, Item.BUTTON);
        mainView[2] = new StringItem(null, MENU_BUY_TICKET, Item.BUTTON);
       
        for (int i = 0; i < mainView.length; i++) {
        	mainView[i].setFont(Font.getFont(Font.FACE_SYSTEM,
                    Font.STYLE_PLAIN, Font.SIZE_SMALL));
        	mainView[i].setLayout(Item.LAYOUT_EXPAND);
        	mainView[i].setDefaultCommand(mainSelectCmd);
        	mainView[i].addCommand(mainExitCmd);
        	mainView[i].setItemCommandListener(this);
            mainForm.append(mainView[i]);
        }

		errorForm = new Form(ERROR_TITLE);
		tracelines = new StringItem(null, "Trace:\n");
		
		busy = new BusyAlert(BUSY_TITLE, this);
		purchasing = new BusyAlert("Purchase", this);
		secureChip = new SecureChipHandler(this);
		
		display.setCurrent(mainForm);
	}
	
	/**
	 * Pauses the application
	 */
	protected void pauseApp() {

	}

	/**
	 * Exits the application
	 * 
	 * @param arg0 Boolean variable demanded by J2ME
	 */
	public void destroyApp(boolean arg0){
		 try {
	            if (msgConn != null)
	            	msgConn.close();
	            if (secureChip.coverConnection != null)
	            	secureChip.coverConnection.disconnect();
	        } catch (Exception e) {
	            trace("Exception DA: " + e.toString());
	            showTrace();
	        }
	        notifyDestroyed();
	    }
	/**
	 * Starts listening for sms
	 */
	public void run(){
		receiveSms();
	}
	
	/**
	 * Listens for incoming sms at portnumber from application properites.
	 * Default portnumber 7552 used if SmsPortAttribute undefined in JAD file.
	 */
	private void receiveSms(){
		trace("receiveSms");
		smsPort = this.getAppProperty(SmsPortAttribute);
        if (smsPort == null) {
            smsPort = DefaultSmsPort;
        }
        
        try{
        	msgConn = (MessageConnection) Connector.open("sms://:" + smsPort);
        	msgConn.setMessageListener(this);
        } catch(Exception e){
        	 trace("Exception LI: " + e.toString());
             showTrace();
        }	
	}
	
	/**
	 * Receives the incoming sms and extracts the payload.
	 * 
	 * @param conn The connection receiving the sms
	 */
	public void notifyIncomingMessage(MessageConnection conn){
		trace("notifyIncomingMessage"); 
		Message msg = null;
		
		if(conn != msgConn)
			return;
		
		
		
		try{
			
			msg = msgConn.receive();
			busy.doAlert(BUSY_RECEIVING_DATA);
			
			manager.playSequence(vibeSequence);
			busy.setText("receiving ticket");
			
			if (msg instanceof TextMessage){
				trace(ERROR_TITLE +" "+ ERROR_RECEIVED_SMS_TXT);
				showTrace();
				return;
			}
			
			byte[] b = ((BinaryMessage) msg).getPayloadData();
			
			if (buf != null && buf.length > 0) {
                byte[] t = new byte[buf.length + b.length];

                System.arraycopy(b, 0, t, 0, b.length);
                System.arraycopy(buf, 0, t, b.length, buf.length);

                buf = t;
            } else {
                buf = b;
            }
			
			String data = getHexString(b);
			StringItem t = new StringItem(ERROR_DATA, data);
			errorForm.append(t);
			
			//Bitwise AND
			if((b[0] & 0xFF) == 0xFF) {
				StringItem s = new StringItem(ERROR_SMS, ERROR_SMS_TICKET);
				errorForm.append(s);
				//showMain();
				busy.setText(BUSY_TICKET_RECEIVED);
	            secureChip.writeMifare(buf);
			} else {
				StringItem s = new StringItem(ERROR_SMS, ERROR_SMS_NOT_A_TICKET);
				errorForm.append(s);
				showMain();
			}
			
		} catch(Exception e){
			trace("Exception NO: " + e.toString());
            showTrace();
		} 
	}
	
	/**
	 * Converts a byte array to string representation
	 * 
	 * @param data The byte array to 
	 * @return the string representation of the byte array
	 */
	private static String getHexString(byte[] data) {
        StringBuffer sb = new StringBuffer();
        
        for (int i = 0; i < data.length; i++) {
            String bs = Integer.toHexString(data[i] & 0xFF);
            if (bs.length() == 1) sb.append(0);
            sb.append(bs);
            sb.append((i + 1) % 4 == 0 ? '\n' : ' ');
        }
        
        return sb.toString();
    }
	
	/**
	 * Presents the error form on the display
	 */
	public void showMain() {
		 display.setCurrent(errorForm);
	 }
	
	/**
	 * Collects strings for tracing.
	 * 
	 * @param line The string to be used for tracing
	 */
	
	public void trace(String line) {
        if (tracelines != null) {
            tracelines.setText(tracelines.getText() + line + "\n");
        } else {
            System.out.println(line);
        }
    }
	
	/**
	 * Notifies the user of an error
	 * Presents the error form if user selects yes to do so.
	 */
	public void showTrace() {
		handle_error = true;
        errorForm.deleteAll();
        errorForm.append(tracelines);
        ConfirmationAlert confAlert = new ConfirmationAlert(CONFIRMATION_ERROR_TITLE, this);
        confAlert.error();
    }
	
	/**
	 * Creates an image from a file
	 * 
	 * @param file The address of the file
	 * @return The file as an Image
	 * @see Image
	 */
	private Image createImage(String file){
		Image img;
		//Constructs the image from the media file
		try{
			img = Image.createImage(file);	
		} catch (Exception e){
			e.printStackTrace();
			img = null;
		}
		return img;
	}
	
	/**
	 * Executes actions based on user commands
	 * 
	 * @param cmd The command issued by the user
	 * @param item The item selected by the user
	 */
	
	 public void commandAction(Command cmd, Item item) {

	        if (cmd == mainExitCmd) {
	            destroyApp(true);
	        } else if (cmd == mainSelectCmd) {
	            mainSelect(item);
	        }else if (cmd == orderBackCmd) {
	            display.setCurrent(mainForm);
	        }else if (cmd == orderSelectCmd) {
	            orderSelect(item);
	        }
	        
	    }
	/**
	 * Performs actions based on user selections in the main menu
	 * 
	 * @param item the item selected by the user 
	 */
	private void mainSelect(Item item) {
            if (item == mainView[0]) {
                //Read current credit
            	info = 0;
            	busy.doAlert(BUSY_CONNECTING_TO_CHIP);
				secureChip.readMifare();
            } else if (item == mainView[1]) {
                //Read last purchase
            	info = 1;
            	busy.doAlert(BUSY_CONNECTING_TO_CHIP);
				secureChip.readMifare();
            } else if (item == mainView[2]) {
            	//Order ticket 
            	placeOrder();
            }
    }
	
	/**
	 * Performs actions based on user selections in the order menu
	 * 
	 * @param item The item selected by the user 
	 */
	private void orderSelect(Item item) {
		String value = "";
        if (item == orderView[0]) {
        	//value = 100
        	value = "%00%64";
        } else if (item == mainView[1]) {
            //value = 200
        	value = "%00%C8";
        } else if (item == mainView[2]) {
        	//value = 500
        	value = "%01%F4";
        }
    	busy.doAlert("Processing purchase...");
    	new HTTPConnection(this, value);
	}
	
	/**
	 * Notifies the user that a purchase has been made
	 * 
	 * @param s The notification to the user
	 */
	public void purchase(String s){
		Alert alert = new Alert("Purchase");
		alert.setString(s);
		alert.setTimeout(1000 * 2);
		manager.setBacklight(1000 * 3);
		manager.playSequence(vibeSequence);
   	 	display.setCurrent(alert, mainForm);
	}
	
	/**
	 * Creates the order form and adds listener to the buttons
	 */
	public void placeOrder(){
		orderForm = new Form(ORDER_MAIN_TITLE);
		orderSelectCmd = new Command(ORDER_CMD_SELECT, Command.ITEM, 1);
        orderBackCmd = new Command(ORDER_CMD_BACK, Command.EXIT, 1);
        
        //Create and add the buttons shown in the main form.
        orderView = new StringItem[3];
        orderView[0] = new StringItem(null, "100"+ENDING_KR, Item.BUTTON);
        orderView[1] = new StringItem(null, "200"+ENDING_KR, Item.BUTTON);
        orderView[2] = new StringItem(null, "500"+ENDING_KR, Item.BUTTON);
       
        orderForm.append("Please select amount:\n");
        
        for (int i = 0; i < orderView.length; i++) {
        	orderView[i].setFont(Font.getFont(Font.FACE_SYSTEM,
                    Font.STYLE_PLAIN, Font.SIZE_SMALL));
        	orderView[i].setLayout(Item.LAYOUT_EXPAND);
        	orderView[i].setDefaultCommand(orderSelectCmd);
        	orderView[i].addCommand(orderBackCmd);
        	orderView[i].setItemCommandListener(this);
            orderForm.append(orderView[i]);
        }
        display.setCurrent(orderForm);
	}
}
